unit SageData;

interface
Uses Windows, Messages,SysUtils;

Const
     CREATOR_ID=                    $5A6E;
     MODULE_NOT_FOUND=              -999;


     EXIT_AND_RERUN=                $F9;
     SAGE_ENABLE=                   WM_USER + 6;
     SAGE_DISABLE=                  WM_USER + 7;
     SAGE_GETSTATUS=                WM_USER + 8;
     ENABLE_AGENT=                  1;
     DISABLE_AGENT=                 2;
     GET_AGENT_STATUS=              3;
     MAXPATH=                       267;                     //267
     MAXSETTINGS=                   256;                     //256
     MAXCOMMANDLINE=                (MAXPATH+MAXSETTINGS);
     MAXCOMMENT=                    256;                     //256
     CB_RESERVED=                   33;                      //33

     STATUS_NOTRUNNING=             0;   // Task is not running.
     STATUS_RUNNING=                1;   // Task is currently running.
     STATUS_COMPLETE=               2;   // Task is not running, but has run in the
                                         // currently scheduled period.
     STATUS_QUEUEDEXEC=             3;   // Task is just about to execute.

     BIT_TASKFLAG_LOGGIN=           1;   // 1 => log execution info to the System Agent log file.
     BIT_TASKFLAG_TASKHASRUN=       3;   // 1 => the task has run at least once before;
                                         // 0 => means it has never run (read only)
     BIT_TASKFLAG_NOIDLETERMINATED= 6;   // 1 => task was terminated because of loss of idle
     BIT_TASKFLAG_DONTRUN=          7;   // 1 => don't run during this period after change task (see below)


     ERROR_TASK_INVALID2=         $558;     //Error in TaskInfo length

     ERROR_CANNOT_ADD_TASK=       -12;   // Unable to add a task, perhaps
                                         //  because out of memory.
     ERROR_AGENT_BUSY=            -13;   // System has timed out the API call.
     ERROR_TASK_NOT_PRESENT=      -14;   // Task_Identifer reference not in task
                                         //  list.
     ERROR_AGENT_NOT_PRESENT=     -15;   // System Agent is not active.
     ERROR_CANNOT_LOAD_DLL=       -16;   // Error loading the DLL.
     ERROR_CANNOT_LOCK_TASK=      -17;   // Task cannot be locked at this time.
     ERROR_TASK_ALREADY_LOCKED=   -18;   // Attempt to lock task failed; another
                                         //  task owns lock.
     ERROR_CANNOT_UNLOCK_TASK=    -19;   // Unable to unlock task, perhaps
                                         //  because not locked.
     ERROR_TASK_LOCKED=           -20;   // Task is locked by a different task,
                                         //  access denied.
     ERROR_WRONG_VERSION=         -21;   // Version of System Agent is not
                                         //  compatible.
     ERROR_TASK_NOT_LOCKED=       -22;   // Attempt to modify a task that is not
                                         //  locked.
     ERROR_TASK_VOLATILE=         -23;   // Cannot volatile lock because task is
                                         //  running.
     ERROR_TASK_INVALID=          -24;   // Data in task is invalid or corrupt.
     ERROR_STALE_DATA=            -25;   // Attempt to modify task with data got
                                         //  before lock.

     ERROR_SAGE_NO_INITIALIZED=  -255;   // SAGE not itilialized yet

type
    TaskInfo = packed record
        StructureSize           : Cardinal;     {Length of TaskInfo}
        Task_Identifier         : Cardinal;     {Task ID}
        Sub_Task_Identifier     : Cardinal;     {Reserved, must be zero}
        Status                  : Cardinal;     {STATUS_XXX Const}
        Result                  : Cardinal;     {result code returned from the last execution of the task}
        Time_Granularity        : Cardinal;     {If non-zero, this field indicates that the task should
                                                 be executed periodically during its scheduled time at an
                                                 interval of Time_Granularity seconds. If the task is still
                                                 running when the next Time_Granularity seconds have
                                                 elapsed, the task is not run; otherwise it is}
        StopAfterTime           : Cardinal;     {If non-zero, this field indicates that the task should
                                                 be terminated after running for StopAfterTime seconds of
                                                 executio}
        ReservedLong            : Cardinal;     {Reserved, should be zero}
        User_Idle               : Cardinal;     {If non-zero, this task indicates that the task should
                                                 not be executed unless User_Idle seconds have elapsed
                                                 with no keyboard or mouse activity. Thus, if a task is
                                                 scheduled to run between 4:00 and 5:00 with User_Idle
                                                 set to 10 minutes (600 seconds), and the user last moved
                                                 his mouse at 3:55, the task will execute at 4:05 unless
                                                 the user interacts with the computer between those
                                                 times, in which case the time countdown will be
                                                 restarted at 10 minutes}
        Powered                 : Cardinal;     {This field indicates that the task should not be run if
                                                 the computer is currently being run on batteries. This
                                                 works only on fully APM-compliant Plug and Play
                                                 machines. Valid values are zero or one (one indicates
                                                 the task should not run on batteries).}
        CreatorId               : Cardinal;     {This field is reserved for use by the task creator.
                                                 The task creator is the application that called
                                                 System_Agent_Add_Task. This can be any value.}
        BeginTime               : TSystemTime;  {The StartTime and EndTime together define the basic
                                                 schedule for a task. The StartTime field indicates the
                                                 beginning of the task's execution period. This can be an
                                                 exact time (in the case of a run-once task), or it can be
                                                 a repeating schedule such as Daily, Monthly, Weekly, or
                                                 Hourly. To specify a repeating schedule, set the units at
                                                 which you wish to repeat to -1. All fields of higher
                                                 precedence must also be set to -1. Precedence order is:
                                                 wYear, wMonth, wDay, wHour, wMinute, wSeconds. The
                                                 wMillisecond field is not honored when scheduling tasks.
                                                 Note wDayOfWeek does not have precedence and should be
                                                 set to -1 unless you are specifically scheduling a task
                                                 to run on a fixed day of the week.}
        EndTime                 : TSystemTime;  {This field specifies the ending time for the period in
                                                 which the task is run. It has the same format as the
                                                 StartTime, and should match its repeating schedule type.
                                                 This field defines the latest time at which the task will
                                                 be started if the task runs only when the machine is
                                                 idle, and it defines the end of the period in which it
                                                 will be started if it has a Time_Granularity set. Also,
                                                 this field defines the time that task will be terminated
                                                 if TerminateAtRangeEnd is set. If no EndTime definition
                                                 is needed, all fields may be set to -1.}
        LastRunStart            : TSystemTime;  {This field indicates the time at which this task was last
                                                 executed. This field should not be changed by the client.}
        LastRunEndScheduled     : TSystemTime;  {This field indicates the specific time the task's period
                                                 is/was scheduled to end according to the schedule defined
                                                 by StartTime and EndTime. This field does not indicate at
                                                 what time the task actually terminated (see
                                                 LastTerminationTime), nor will be terminated, but rather
                                                 at what time the period for this task is scheduled to end.}
        LastTerminationTime     : TSystemTime;  {This field contains the specific time this task was last
                                                 terminated.}
        ReservedTime1           : TSystemTime;  {This field is used internally by System Agent, and should
                                                 not be used or modified by the client.}
        LastAlarmTime           : TSystemTime;  {This field indicates the specific time the last scheduled
                                                 program notification was displayed for this task. This
                                                 field should not be used by the client.}
        ReservedTime2           : TSystemTime;  {These fields are reserved for future use. They should}
        ReservedTime3           : TSystemTime;  {remain zero and should not be used by the client.}
        StartupInfo             : TStartupInfo; {This field defines the STARTUPINFO structure that should
                                                 be passed to CreateProcess when the task is started. See
                                                 CreateProcess.}
        dwProcessId             : DWORD;        {This field defines the Process Id for the task, if it is
                                                 running. See CreateProcess. This field should not be
                                                 modified by the client.}
        dwThreadId              : DWORD;        {This field defines the Thread Id for the task, if it is
                                                 running. See CreateProcess. This field should not be
                                                 modified by the client.}
        LockingProcess          : DWORD;        {This field indicates the Process ID of the program who
                                                 owns the lock on this task, if there is one. This field
                                                 should not be modified by the client.}
        LockTime                : Cardinal;     {This field records the GetTickCount return value when the
                                                 task was locked. This field is used for data validation,
                                                 and should not be used by the client.}
        fdwCreate               : DWORD;        {This field indicates the fdwCreate flags passed to
                                                 CreateProcess. If zero, System Agent uses
                                                 CREATE_NEW_CONSOLE | CREATE_NEW_PROCESS_GROUP |
                                                 NORMAL_PRIORITY_CLASS. See CreateProcess.}
        taskflags               : DWORD;        {Bit flags. See Const BIT_TASKFLAGS }
        SystemTask              : char;         {0 means standard task. 1 means task is a system task,
                                                 and will not show up in the System Agent window.}
        TerminateAtRangeEnd     : char;         {If this field is set to 1, the task will be terminated at
                                                 the EndTime if it is still running.}
        StartupTask             : char;         {If this field is set to 1, the task will be run only when
                                                 System Agent engine is started}
        AlarmEnabled            : char;         {If this field is set to 1, and a task does not run during
                                                 its scheduled period for any reason, System Agent will put
                                                 up a dialog box to tell the user. Notification will only
                                                 occur if SAGE is running. Tasks can fail to run because
                                                 the machine was off, the system was never idle, or the
                                                 system was running on batteries during the scheduled time.
                                                 If the task runs during a subsequent period and the dialog
                                                 has not been not yet posted, the alarm is cancelled.}
        RunNow                  : char;         {If this field is set to 1, the task will execute
                                                 immediately}
        TerminateNoIdle         : char;         {If this field is set to 1, the task will be terminated if
                                                 it is running and the user uses the keyboard or mouse.}
        Disabled                : char;         {If this field is set to 1, the task is disabled, and will
                                                 not be scheduled to run}
        TerminateNow            : Char;         {If this field is set to 1, the task will be terminated if
                                                 it is currently runnin}
        RestartNoIdle           : char;         {If this field is set to 1, the task will restart after
                                                 User_Idle seconds if it was terminated because
                                                 TerminateNoIdle was set and the user use the mouse or
                                                 keyboard.}
        CommandLine             : array[0..MAXCOMMANDLINE -1] of char;   {This field defines the full command
                                                                       line for the task. The task may be
                                                                       a .COM, .BAT, .EXE, or associated
                                                                       file type. This is the command line
                                                                       that is passed to CreateProcess. It
                                                                       must be zero-terminated.}
        Comment                 : array[0..MAXCOMMENT -1] of char;       {This is the description field shown
                                                                       in the System Agent window. If this
                                                                       field is all zeros, the System Agent
                                                                       will use the CommandLine instead}
        WorkingDirectory        : array[0..MAXPATH -1] of char;          {This is the working directory as
                                                                       defined by CreateProcess.}
        Reserved                : array[0..CB_RESERVED -1] of char;      {This field is reserved for use by
                                                                       the System Agent and future API
                                                                       enhancements. It should not be used
                                                                       or modified.}
    end;

const
     MaxTaskItems  = ( (MaxInt div 2 ) div SizeOf(integer) );

type
    TaskArray = array[0..SizeOf(TaskInfo)] of TaskInfo;
    PTaskInfo = ^TaskArray;
    PTask     = ^TaskInfo;


implementation

end.
