unit SageUnit;

{  TSAGE 0.7   -   1997 by Nelson Fernandez

   First, sorry about my english. Spanish is my first languaje.

   This component is freeware. Feel free to modify and enhance it, but PLEASE
   don't redistribute modificated versions. Email me the modification to upgrade the source
   and have only one version of the component.
   This component don't have yet a documentation, but the methods are easy to
   use, there are a demo program, and in the file SAGEAPI.HTML is the rest.
   In SAGEAPI.HTML there are some documentation about how configure programs from SAGE, how
   parse the command line for parameters, how process multiple configurations and
   returns codes specification.
   This text can be download from:
   http://www.microsoft.com/win32dev/guidelns/sageapi.htm

   There is a patch for Sage.Dll in:
   http://www.microsoft.com/kb/articles/q140/4/79.htm

   For run the Demo program, there isn't need to install the component. Compile SageTest and
   presto! Then, if you want to install it, copy SageData.pas, SageUnit.pas and SageUnit.res
   to your components directory and compile.

   If you have any question or suggestion, please email me to: xtzacd1@impsat1.com.ar

   History
          11/06/97  v0.1 - prebeta version. Home test.
          05/07/97  v0.6 - beta version. Public release
          29/08/97  v0.7 - Now the Sage.Dll is linked at runtime and the functions
                           are allocated on demand. (Tip from Pekka E. Haapalainen)

   -------------
   Nelson Fernandez
   Avda. Gaona 2143 1ro I
   Buenos Aires, Ramos Mejia (1704)
   Argentina.
   Email: xtzacd1@impsat1.com.ar
}

interface

uses
  Windows, Dialogs, Classes, SageData;

type
  TSage = class( TComponent )
  private
    { Private declarations }
    fDetected       : integer;
    fInitialized    : boolean;
    fChanged        : BOOL;
    fNumTask        : integer;
    fpTaskList      : PTaskInfo;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create( AOwner: TComponent ); override;
    destructor  Destroy; override;
    function Enable: integer;
    function Disable: integer;
    function GetStatus: integer;
    function Initialize: integer;
    function GetTaskList: integer;
    function LockTask( TaskId: cardinal; Volatile: BOOL ): integer;
    function UnLockTask( TaskId: cardinal; Reenable: BOOL ): integer;
    function AddTask( var Task: TaskInfo; var TaskId: cardinal ): integer;
    function ChangeTask( Task: TaskInfo; TaskId: cardinal ): integer;
    function RemoveTask( TaskId: cardinal ): integer;
    function SearchTask( TaskId: cardinal ): integer;
    property pTaskList: PTaskInfo read fpTaskList;  // List of Task in SAGE
  published
    { Published declarations }
    property Detected : integer   read fDetected;  // Is SAGE loaded?
    property Changed  : BOOL      read fChanged;   // TaskList Changed from last call to GetTaskList?
    property NumTask  : integer   read fNumTask;   // # of Task in SAGE
  end;

procedure Register;
procedure GenError( FkError: integer );
procedure EmptyTask( var Task: TaskInfo );

implementation
   Uses SysUtils;

{================================ SAGE API ====================================}
type
    PFNSystemAgentDetect = ^TFNSystemAgentDetect;
    TFNSystemAgentDetect = function : integer stdcall;

    PFNSystemAgentEnable = ^TFNSystemAgentEnable;
    TFNSystemAgentEnable = function( Enablefunc: integer ): Integer stdcall;

    PFNSystemAgentInitialize = ^TFNSystemAgentInitialize;
    TFNSystemAgentInitialize = function : integer stdcall;

    PFNSystemAgentEnd = ^TFNSystemAgentEnd;
    TFNSystemAgentEnd = function : integer stdcall;

    PFNSystemAgentGetTaskList = ^TFNSystemAgentGetTaskList;
    TFNSystemAgentGetTaskList = function( var pTaskList: PTaskInfo; var changed : BOOL ): integer stdcall;

    PFNSystemAgentAddTask = ^TFNSystemAgentAddTask;
    TFNSystemAgentAddTask = function( Task: PTask; var Task_Identifier: cardinal ): integer stdcall;

    PFNSystemAgentRemoveTask = ^TFNSystemAgentRemoveTask;
    TFNSystemAgentRemoveTask = function( Task_Identifier: cardinal ): integer stdcall;

    PFNSystemAgentLockTask = ^TFNSystemAgentLockTask;
    TFNSystemAgentLockTask = function( Task_Identifier: cardinal; dwProcessId: DWORD; volatile: BOOL ): integer stdcall;

    PFNSystemAgentUnlockTask = ^TFNSystemAgentUnlockTask;
    TFNSystemAgentUnlockTask = function( Task_Identifier: cardinal; dwProcessId: DWORD; reenable: BOOL ): integer stdcall;

    PFNSystemAgentChangeTask = ^TFNSystemAgentChangeTask;
    TFNSystemAgentChangeTask = function( Task: PTask; Task_Identifier: cardinal ): integer stdcall;

var
  SAGEDLL    : string  = 'SAGES.DLL';
  SAGEModule : HModule = 0;

  SA_Detect      : TFNSystemAgentDetect      = nil;
  SA_Enable      : TFNSystemAgentEnable      = nil;
  SA_Initialize  : TFNSystemAgentInitialize  = nil;
  SA_End         : TFNSystemAgentEnd         = nil;
  SA_GetTaskList : TFNSystemAgentGetTaskList = nil;
  SA_AddTask     : TFNSystemAgentAddTask     = nil;
  SA_RemoveTask  : TFNSystemAgentRemoveTask  = nil;
  SA_LockTask    : TFNSystemAgentLockTask    = nil;
  SA_UnlockTask  : TFNSystemAgentUnlockTask  = nil;
  SA_ChangeTask  : TFNSystemAgentChangeTask  = nil;

{------------------------------------------------------------------------------}
constructor TSage.Create( AOwner: TComponent );
begin
     inherited Create( AOwner );

     SAGEModule := LoadLibrary( pChar( SAGEDLL ) );
     If SAGEModule = 0 then
        fDetected := MODULE_NOT_FOUND

     else begin
        if @SA_Detect = nil then
           @SA_Detect := GetProcAddress( SAGEModule, 'System_Agent_Detect' );
        if @SA_Detect <> nil then
           fDetected := SA_Detect;
     end;

     fInitialized := FALSE;
end;

{------------------------------------------------------------------------------}
destructor TSage.Destroy;
begin
     if fInitialized then begin
        if @SA_End = nil then
           @SA_End := GetProcAddress( SAGEModule, 'System_Agent_End' );
        if @SA_End <> nil then
           SA_End;
     end;

     inherited Destroy;
end;

{------------------------------------------------------------------------------}
function TSage.Enable: integer;
begin
     Result := 0;
     if ( fDetected > 0 ) then begin
        if @SA_Enable = nil then
           @SA_Enable := GetProcAddress( SAGEModule, 'System_Agent_Enable' );
        if @SA_Enable <> nil then
           Result := SA_Enable( ENABLE_AGENT );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.Disable: integer;
begin
     Result := 0;
     if ( fDetected > 0 ) then begin
        if @SA_Enable = nil then
           @SA_Enable := GetProcAddress( SAGEModule, 'System_Agent_Enable' );
        if @SA_Enable <> nil then
           Result := SA_Enable( DISABLE_AGENT );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.GetStatus: integer;
begin
     Result := 0;
     if ( fDetected > 0 ) then begin
        if @SA_Enable = nil then
           @SA_Enable := GetProcAddress( SAGEModule, 'System_Agent_Enable' );
        if @SA_Enable <> nil then
           Result := SA_Enable( GET_AGENT_STATUS );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.Initialize : integer;
begin
     Result := -1;      // check this

     if @SA_Initialize = nil then
        @SA_Initialize := GetProcAddress( SAGEModule, 'System_Agent_Initialize' );
     if @SA_Initialize <> nil then
        Result := SA_Initialize;

     if Result = 0 then
        fInitialized := TRUE;
end;

{------------------------------------------------------------------------------}
function TSage.GetTaskList: integer;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;

     if fInitialized then begin
        if @SA_GetTaskList = nil then
           @SA_GetTaskList := GetProcAddress( SAGEModule, 'System_Agent_Get_Task_List' );
        if @SA_GetTaskList <> nil then begin
           Result := SA_GetTaskList( fpTaskList, fChanged );
           fNumTask := Result;
        end;
     end;
end;

{------------------------------------------------------------------------------}
function TSage.LockTask( TaskID: cardinal; Volatile: BOOL ): integer;
var
   ProcID : DWORD;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;
     if fInitialized then begin
        if @SA_LockTask = nil then
           @SA_LockTask := GetProcAddress( SAGEModule, 'System_Agent_Lock_Task' );
        if @SA_LockTask <> nil then begin
           ProcID := GetCurrentProcessId();
           Result := SA_LockTask( TaskID, ProcID, Volatile );
           fNumTask := GetTaskList;
        end;
     end;
end;

{------------------------------------------------------------------------------}
function TSage.UnLockTask( TaskId: cardinal; Reenable: BOOL ): integer;
var
   ProcID : DWORD;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;
     if fInitialized then begin
        if @SA_UnlockTask = nil then
           @SA_UnlockTask := GetProcAddress( SAGEModule, 'System_Agent_Unlock_Task' );
        if @SA_UnlockTask <> nil then begin
           ProcID := GetCurrentProcessId();
           Result := SA_UnlockTask( TaskID, ProcID, Reenable );
        end;
     end;
end;

{------------------------------------------------------------------------------}
function TSage.RemoveTask( TaskId: cardinal ): integer;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;
     if fInitialized then begin
        if @SA_RemoveTask = nil then
           @SA_RemoveTask := GetProcAddress( SAGEModule, 'System_Agent_Remove_Task' );
        if @SA_RemoveTask <> nil then
           Result := SA_RemoveTask( TaskID );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.AddTask( var Task: TaskInfo; var TaskId: cardinal ): integer;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;
     if fInitialized then begin
        if @SA_AddTask = nil then
           @SA_AddTask := GetProcAddress( SAGEModule, 'System_Agent_Add_Task' );
        if @SA_AddTask <> nil then
           Result := SA_AddTask( @Task, TaskID );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.ChangeTask( Task: TaskInfo; TaskId: cardinal ): integer;
begin
     Result := ERROR_SAGE_NO_INITIALIZED;
     if fInitialized then begin
        if @SA_ChangeTask = nil then
           @SA_ChangeTask := GetProcAddress( SAGEModule, 'System_Agent_Change_Task' );
        If @SA_ChangeTask <> nil then
           Result := SA_ChangeTask( @Task, TaskID );
     end;
end;

{------------------------------------------------------------------------------}
function TSage.SearchTask( TaskID: cardinal ): integer;
var
   nLoop : integer;
begin
     Result := -1;
     for nLoop := 0 to ( fNumtask-1 ) do
         If pTaskList[ nLoop ].Task_Identifier = TaskID then begin
            Result := nLoop;
            Break;
         end;
end;

{============================ Support Functions ===============================}
{------------------------------------------------------------------------------}
procedure EmptyTask( var Task: TaskInfo );
Begin
     FillChar( Task, SizeOf( Task ), 0 );
     With Task do begin
          StructureSize   := SizeOf( TaskInfo );
          CreatorId       := CREATOR_ID;
          StartUpInfo.cb  := SizeOf( TStartupInfo );
     end;
end;

{------------------------------------------------------------------------------}
procedure GenError( FkError: integer );
var
   cMsg : string;
begin
     case FkError of
          ERROR_CANNOT_ADD_TASK      : cMsg := 'Unable to add a Task';
          ERROR_AGENT_BUSY           : cMsg := 'System Agent is Busy';
          ERROR_TASK_NOT_PRESENT     : cMsg := 'Task not present';
          ERROR_AGENT_NOT_PRESENT    : cMsg := 'System Agent is not Active';
          ERROR_CANNOT_LOAD_DLL      : cMsg := 'Error loading the DLL';
          ERROR_CANNOT_LOCK_TASK     : cMsg := 'Task cant be locked at this time';
          ERROR_TASK_ALREADY_LOCKED  : cMsg := 'Attempt to lock task failed; another task owns lock';
          ERROR_CANNOT_UNLOCK_TASK   : cMsg := 'Unable to unlock task, perhaps because not locked';
          ERROR_TASK_LOCKED          : cMsg := 'Task is locked by a different task, access denied';
          ERROR_WRONG_VERSION        : cMsg := 'Version of System Agent is not compatible';
          ERROR_TASK_NOT_LOCKED      : cMsg := 'Attempt to modify a task that is not locked';
          ERROR_TASK_VOLATILE        : cMsg := 'Cannot volatile lock because task is running';
          ERROR_TASK_INVALID         : cMsg := 'Data in task is invalid or corrupt';
          ERROR_STALE_DATA           : cMsg := 'Attempt to modify task with data got before lock';
          ERROR_SAGE_NO_INITIALIZED  : cMsg := 'SAGE not Initialized yet';
          ERROR_TASK_INVALID2        : cMsg := 'Error in TaskInfo length';
     else
          cMsg := 'Undefined Error: ' + IntToStr( FkError );
     end;

     MessageBeep( MB_ICONHAND );
     MessageDlg( cMsg, mtError, [mbOk], 0 );

end;


procedure Register;
begin
  RegisterComponents('System', [TSage]);
end;

end.
